/*-
 * Memory usage:
 *
 * 0x02000000 - 0x0203ffff : EWRAM (256K)
 * 0x03000000 - 0x03007fff : IWRAM (32K)
 * 0x04000000 - 0x040003ff : I/O Register (1K)
 * 0x05000000 - 0x050003ff : Palette RAM (1K)
 * 0x06000000 - 0x06017fff : VRAM (96K)
 *
 * Note:
 */

OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
OUTPUT_ARCH(arm)
ENTRY(start)

MEMORY
{
  /* general internal memory */
  bios_system_rom (x)   : ORIGIN = 0x00000000, LENGTH = 0x00004000
  ewram           (awx) : ORIGIN = 0x02000000, LENGTH = 0x00040000
  iwram           (awx) : ORIGIN = 0x03000000, LENGTH = 0x00008000
  /* 0x03007f00 - 0x03007fff is reserved by GBA BIOS */
  io_registers    (w)   : ORIGIN = 0x04000000, LENGTH = 0x00000400

  /* display internal memory */
  bg_obj_palette  (aw)  : ORIGIN = 0x05000000, LENGTH = 0x00000400
  vram            (aw)  : ORIGIN = 0x06000000, LENGTH = 0x00018000
  obj_attributes  (aw)  : ORIGIN = 0x07000000, LENGTH = 0x00000400

  /* external memory */
  cartridge       (rx)  : ORIGIN = 0x08000000, LENGTH = 0x02000000
}

SECTIONS 
{
	.text : 
        {
                *startup.o(.text)
		*(.text)
	} AT>cartridge

	.rodata . : 
        {
		*(.rodata)
	} AT>cartridge

        .data :
        {
                *(.data)
	} >ewram AT>cartridge

        .bss :
        {
		*(.bss)
	} >ewram AT>cartridge
}

__stack = ORIGIN(iwram) + LENGTH(iwram) - 0x400;

__lma_data_beg = LOADADDR(.data);

__vma_data_beg = ADDR(.data);
__vma_data_end = __vma_data_beg + SIZEOF(.data);

__vma_bss_beg  = ADDR(.bss); 
__vma_bss_end  = __vma_bss_beg + SIZEOF(.bss);

end = ORIGIN(iwram);